@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package Test::Mojo::Role::OpenAPI::Modern; # git description: v0.001-6-g16c84a5
+package Test::Mojo::Role::OpenAPI::Modern; # git description: v0.002-5-g34097db
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Test::Mojo role providing access to an OpenAPI document and parser
 # KEYWORDS: validation evaluation JSON Schema OpenAPI Swagger HTTP request response
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 use 5.020;  # for fc, unicode_strings features
 use strictures 2;
@@ -39,15 +39,37 @@ has 'openapi' => sub ($self) {
   die 'openapi object or configs required';
 };
 
+sub _openapi_stash ($self, @data) { Mojo::Util::_stash(_openapi_stash => $self, @data) }
+
+after _request_ok => sub ($self, @args) {
+  $self->{_openapi_stash} = {};
+};
+
 sub request_valid ($self, $desc = 'request is valid') {
-  my $result = $self, $self->openapi->validate_request($self->tx->req);
-  return $self->test('ok', $result, $desc);
+  return $self->test('ok', $self->request_validation_result, $desc);
 }
 
 sub response_valid ($self, $desc = 'response is valid') {
+  return $self->test('ok', $self->response_validation_result, $desc);
+}
+
+sub request_validation_result ($self) {
+  my $result = $self->_openapi_stash('request_result');
+  return $result if $result;
+
+  $result = $self->openapi->validate_request($self->tx->req);
+  $self->_openapi_stash(request_result => $result);
+  return $result;
+}
+
+sub response_validation_result ($self) {
+  my $result = $self->_openapi_stash('response_result');
+  return $result if $result;
+
   my $options = { request => $self->tx->req };
-  my $result = $self->openapi->validate_response($self->tx->res, $options);
-  return $self->test('ok', $result, $desc);
+  $result = $self->openapi->validate_response($self->tx->res, $options);
+  $self->_openapi_stash(response_result => $result);
+  return $result;
 }
 
 1;
@@ -64,7 +86,7 @@ Test::Mojo::Role::OpenAPI::Modern - Test::Mojo role providing access to an OpenA
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -104,8 +126,9 @@ version 0.002
                       const: ok
   YAML
 
-  my $t = Test::Mojo->new('MyApp', { ... })
+  my $t = Test::Mojo
     ->with_roles('+OpenAPI::Modern')
+    ->new('MyApp', { ... })
     ->openapi($openapi);
 
   $t->post_ok('/foo/hello')
@@ -144,6 +167,20 @@ L<Mojolicious::Plugin::OpenAPI::Modern/validate_request>, producing a boolean te
 Passes C<< $t->tx->res >> to L<OpenAPI::Modern/validate_response> as in
 L<Mojolicious::Plugin::OpenAPI::Modern/validate_response>, producing a boolean test result.
 
+=head2 request_validation_result
+
+Returns the L<JSON::Schema::Modern::Result> object for the validation result for the last request,
+or calculates it if not already available.
+
+Does not emit a test result.
+
+=head2 response_validation_result
+
+Returns the L<JSON::Schema::Modern::Result> object for the validation result for the last response,
+or calculates it if not already available.
+
+Does not emit a test result.
+
 =head1 FUTURE FEATURES
 
 Lots of features are still to come, including:
